package cz.atos.Selenium;

import com.typesafe.config.Config;
import com.typesafe.config.ConfigFactory;
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.chrome.*;
import org.openqa.selenium.edge.EdgeDriver;
import org.openqa.selenium.ie.InternetExplorerDriver;
import org.openqa.selenium.ie.InternetExplorerOptions;
import org.openqa.selenium.remote.DesiredCapabilities;
import org.testng.annotations.*;
import org.testng.asserts.SoftAssert;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;


import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;


import static cz.atos.Selenium.base.BaseClass.ActionsList;


public class MainPageTest {

    static {
        SimpleDateFormat dateFormat = new SimpleDateFormat("dd-MM-yyyy-HH-mm-ss");
        System.setProperty("current.date.time", dateFormat.format(new Date()));
    }

    private WebDriver driver;
    private SoftAssert softAssertions;
    private Config config;
    private String browser;
    private Logger logger;
    private DesiredCapabilities cap;
    private InternetExplorerOptions ieOptions;

    @BeforeClass
    public void setup() {
        config = ConfigFactory.load();
        softAssertions = new SoftAssert();

        File directory = new File("log");
        if (!directory.exists()) {
            directory.mkdir();
        }

        logger = LogManager.getLogger("MainPageTest");
    }

    @AfterMethod
    public void tearDown() {
        softAssertions.assertAll();
        // zavírání prohližeče
        driver.quit();
    }

    @BeforeMethod
    public void setBrowser() {
        if (config.hasPath("conf.browser") && config.hasPath("conf.browser.name")) {
            browser = config.getString("conf.browser.name");
        } else {
            browser = "IE";
        }
        String version = null;

        if (config.hasPath("conf.browser.version")) {
            version = config.getString("conf.browser.version");
        }

        File file;

        switch (browser) {
            case "chrome":
                file = new File("testConfig/webDriver/chromeDriver/" + version + "/chromedriver.exe");
                System.setProperty("webdriver.chrome.driver", file.getAbsolutePath());

                break;
            case "IE":
                file = new File("testConfig/webDriver/ieDriver/IEDriverServer.exe");
                System.setProperty("webdriver.ie.driver", file.getAbsolutePath());
                ieOptions = new InternetExplorerOptions();
                ieOptions.setCapability("ignoreProtectedModeSettings", true);
                ieOptions.setCapability("nativeEvents", false);
                ieOptions.setCapability("browserFocus", true);
                ieOptions.setCapability("ignoreZoomSetting", true);
                ieOptions.setCapability("requireWindowFocus", true);
                ieOptions.setCapability("unexpectedAlertBehaviour", "accept");
                ieOptions.setCapability("disable-popup-blocking", true);
                ieOptions.setCapability("enablePersistentHover", true);
                ieOptions.setCapability(InternetExplorerDriver.INTRODUCE_FLAKINESS_BY_IGNORING_SECURITY_DOMAINS, true);
                  break;
            case "edge":
                file = new File("testConfig/webDriver/edgeDriver/" + version + "/msedgedriver.exe");
                System.setProperty("webdriver.edge.driver", file.getAbsolutePath());
                break;
            case "edgeIE":
                file = new File("testConfig/webDriver/ieDriver/IEDriverServer.exe");
                System.setProperty("webdriver.ie.driver", file.getAbsolutePath());
                ieOptions = new InternetExplorerOptions();
                ieOptions.attachToEdgeChrome();
                ieOptions.setCapability("ignoreProtectedModeSettings", true);
                ieOptions.setCapability("nativeEvents", false);
                ieOptions.setCapability("browserFocus", true);
                ieOptions.setCapability("ignoreZoomSetting", true);
                ieOptions.setCapability("requireWindowFocus", true);
                ieOptions.setCapability("unexpectedAlertBehaviour", "accept");
                ieOptions.setCapability("disable-popup-blocking", true);
                ieOptions.setCapability("enablePersistentHover", true);
                ieOptions.setCapability(InternetExplorerDriver.INTRODUCE_FLAKINESS_BY_IGNORING_SECURITY_DOMAINS, true);

                File edgeFile = new File("C:\\Program Files (x86)\\Microsoft\\Edge\\Application\\msedge.exe");
                ieOptions.withEdgeExecutablePath(edgeFile.getAbsolutePath());     // Edge Browser directory path

                break;
        }
    }


    @Test
    public void openPage() {
        List<String> scenarios = config.getStringList("conf.scenarios");
        logger.info(scenarios);

        scenarios.forEach((scenario) -> {
            try {
                switch (browser) {
                    case "chrome":
                        driver = new ChromeDriver();
                        break;
                    case "IE":
                        driver = new InternetExplorerDriver(ieOptions);
                        break;
                    case "edge":
                        driver = new EdgeDriver();
                        break;
                    case "edgeIE":
                        driver = new InternetExplorerDriver(ieOptions);
                        break;
                }

                driver.manage().window().maximize();

                Config scConfig = ConfigFactory.load(scenario);
                String[] cfgArr = scenario.split("/");
                String cfgName = cfgArr[cfgArr.length - 1].split("[.]conf")[0];
                List<String> actions = scConfig.getStringList(cfgName + ".actions");

                logger.info("Starting test " + cfgName);
                actions.forEach((action) -> {
                    try {
                        Thread.sleep(scConfig.getInt(cfgName + ".defaultSleep"));
                    } catch (InterruptedException e) {
                        logger.error(e);
                        e.printStackTrace();
                    }
                    ActionsList(driver, browser, scConfig.getConfig(action), softAssertions);
                });
            } catch (Exception ex) {
                logger.error(ex);
                ex.printStackTrace();
            } finally {
                // assertneme vsechny softy k scenari
                softAssertions.assertAll();
                // zavreme prohlizec
                driver.quit();
                String[] cfgArr= scenario.split("/");
                String cfgName = cfgArr[cfgArr.length - 1].split("[.]conf")[0];
                logger.info("End of test " + cfgName);
            }
        });


    }
}
