package cz.atos.Selenium.base;

import com.typesafe.config.Config;
import com.typesafe.config.ConfigValue;
import org.apache.log4j.Logger;
import org.openqa.selenium.By;
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.WebElement;
import org.openqa.selenium.support.ui.ExpectedConditions;
import org.openqa.selenium.support.ui.WebDriverWait;
import org.testng.Assert;
import org.testng.asserts.SoftAssert;

import java.util.List;
import java.util.Map;
import java.util.Set;

public class AssertMethods {

    private static final Logger logger = Logger.getLogger(AssertMethods.class);

    public static void HardAssertValues(WebDriver driver, Config config) {
        logger.info("Hard Assert Values");
        List<String> assertList = config.getStringList("hardAssertValuesList");
        assertList.forEach(assertItem -> {
            String realValue = driver.findElement(By.xpath(config.getString("hardAssertValues." + assertItem + ".input"))).getAttribute("value");
            String expectedValue = config.getString("hardAssertValues." + assertItem + ".expectedValue");
            logger.info( realValue + " - " + expectedValue);
            Assert.assertEquals(realValue, expectedValue, "element: " + config.getString("hardAssertValues." + assertItem + ".input") + " Ocekavana hodnota: " + expectedValue + " Skutecne navracena hodnota: " + realValue +" Chyba:");
        });
        logger.info("Hard Assert Values - END");
    }

    public static void SoftAssertValues(WebDriver driver, Config config, SoftAssert softAssertions) {
        logger.info("Soft Assert Values");
        List<String> assertList = config.getStringList("softAssertValuesList");
        assertList.forEach(assertItem -> {
            String realValue = driver.findElement(By.xpath(config.getString("softAssertValues." + assertItem + ".input"))).getAttribute("value");
            String expectedValue = config.getString("softAssertValues." + assertItem + ".expectedValue");
            logger.info( realValue + " - " + expectedValue);
            softAssertions.assertEquals(realValue, expectedValue, "element:  " +(config.getString("softAssertValues." + assertItem + ".input")) + " Ocekavana hodnota: " + expectedValue + " Skutecne navracena hodnota: " + realValue +" Chyba:");
        });

        logger.info("Soft Assert Values - END");
    }

    public static void SoftAssertImgExists(WebDriver driver, Config config, SoftAssert softAssertions) {
        logger.info("Soft assert of Image presence");
        logger.info(config.getString("element"));
        List<String> typeList = config.getStringList("type");
        StringBuilder sibling = new StringBuilder();
        for (Object o : typeList) {
            sibling.append("/following-sibling::img");
            if (!o.equals("")) {
                logger.info(o.toString());
                softAssertions.assertTrue(driver.findElement(By.xpath(config.getString("element") + "" + sibling )).getAttribute("src").contains(o.toString()),"element: " + config.getString("element") + " nema za sebou ikonku: " + o + " Chyba:" );
            }
        }
        logger.info("Soft assert of Images presence - END");
    }

    //  JR - musim doprobrat s Michalem
    public static void SoftAssertImgExistsByXPath(WebDriver driver, Config config, SoftAssert softAssertions) {
        logger.info("Soft assert of Image presence by XPath");
        logger.info(config.getString("element"));
        String o = config.getString("type");
        if (!o.equals("")) {
            logger.info(o);
            softAssertions.assertTrue(driver.findElement(By.xpath(config.getString("element"))).getAttribute("src").contains(o),"nenalezena ikonka: "  + o + " podle cesty: " + config.getString("element") + " Chyba:" );
        }
        logger.info("Soft assert of Images presence - END");
    }

    public static void SoftAssertPopupDisplayed(WebDriver driver, String popupType, String popupText, SoftAssert softAssertions) {
        logger.info("Soft assert of Popup presence");
        softAssertions.assertTrue(driver.findElement(By.xpath("//div[@class='gma_popup']")).isDisplayed(), "Vyskakovaci okno neni zobrazene. Chyba:");
        if (!"".equals(popupText)) {
            logger.info(" - " + popupText);
            // softAssertions.assertEquals(driver.findElement(By.xpath("//table[@class='"+popupType+"']/thead/td")).getText(),popupText);
        }
        logger.info("Soft assert of Popup presence - END");
    }


    public static void AssertTable (WebDriver driver, Config config) {
        logger.info("AssertTable");
        String elementPath = config.getString("element");
        Config contentConfig = config.getConfig("content");
        Set<Map.Entry<String, ConfigValue>> content = contentConfig.entrySet();
        for (Map.Entry<String, ConfigValue> entry : content) {
            logger.info(entry.getKey() + "/" + entry.getValue().unwrapped().toString());
            List<String> rowContent = (List<String>) contentConfig.getAnyRefList(entry.getKey());
            int entryInt = Integer.parseInt(entry.getKey().replace("\"",""));
            for (int i = 0; i < rowContent.size(); i++) {
                String value = rowContent.get(i);
                logger.info(String.format("checking table item %2d / %2d with value %s", entryInt, i + 1, value));
                if (value != null) {
                    String foundValue = driver.findElement(By.xpath(elementPath + "/tbody/tr[" + entryInt + "]/*[" + (i + 1) + "]")).getText();
                    Assert.assertEquals(value, foundValue);
                }
            }
        }
        logger.info("Soft Assert Table - END");
    }

    public static void SoftAssertTable (WebDriver driver, Config config, SoftAssert softAssertions) {
        logger.info("SoftAssertTable");
        String elementPath = config.getString("element");
        Config contentConfig = config.getConfig("content");
        Set<Map.Entry<String, ConfigValue>> content = contentConfig.entrySet();
        for (Map.Entry<String, ConfigValue> entry : content) {
            logger.info(entry.getKey() + "/" + entry.getValue().unwrapped().toString());
            List<String> rowContent = (List<String>) contentConfig.getAnyRefList(entry.getKey());

            int entryInt = Integer.parseInt(entry.getKey().replace("\"",""));
            for (int i = 0; i < rowContent.size(); i++) {
                String value = rowContent.get(i);
                logger.info(String.format("checking table item %2d / %2d with value %s", entryInt, i + 1, value));
                if (value != null) {
                    String foundValue = driver.findElement(By.xpath(elementPath + "/tbody/tr[" + entryInt + "]/*[" + (i + 1) + "]")).getText();
                    softAssertions.assertEquals(value, foundValue);
                }
            }
        }
    }

    public static void AssertExists (WebDriver driver, String elementPath) {
        logger.info("AssertExists");
        logger.info(String.format("Checking element %s", elementPath));
        Assert.assertTrue(exists(elementPath, driver), String.format("Element %s does not exists", elementPath));
    }

    public static void SoftAssertExists (WebDriver driver, String elementPath, SoftAssert softAssertions) {
        logger.info("SoftAssertExists");
        logger.info(String.format("Checking element %s", elementPath));
        softAssertions.assertTrue(exists(elementPath, driver), String.format("Element %s does not exists", elementPath));
    }

    public static void AssertDisplayed (WebDriver driver, String elementPath) {
        logger.info("AssertDisplayed");
        logger.info(String.format("Checking element %s", elementPath));
        Assert.assertTrue(driver.findElement(By.xpath(elementPath)).isDisplayed(), String.format("Element %s is not displayed", elementPath));
    }

    public static void SoftAssertDisplayed (WebDriver driver, String elementPath, SoftAssert softAssertions) {
        logger.info("SoftAssertDisplayed");
        logger.info(String.format("Checking element %s", elementPath));
        softAssertions.assertTrue(driver.findElement(By.xpath(elementPath)).isDisplayed(), String.format("Element %s is not displayed", elementPath));
    }

    public static void AssertClickable (WebDriver driver, Config elementConfig) {
        logger.info("AssertClickable");
        String elementPath = elementConfig.getString("clickable");
        logger.info(String.format("Checking element %s", elementPath));
        WebElement element = driver.findElement(By.xpath(elementPath));
        int wait = 15;
        if(elementConfig.hasPath("wdw")) {
            wait = elementConfig.getInt("wdw");
        }

        Assert.assertTrue(isClickable(element, driver, wait), String.format("Element %s is not clickable", elementPath));
    }

    public static void SoftAssertClickable (WebDriver driver, Config elementConfig, SoftAssert softAssertions) {
        logger.info("SoftAssertClickable");
        String elementPath = elementConfig.getString("clickable");
        logger.info(String.format("Checking element %s", elementPath));
        WebElement element = driver.findElement(By.xpath(elementPath));
        int wait = 15;
        if(elementConfig.hasPath("wdw")) {
            wait = elementConfig.getInt("wdw");
        }

        softAssertions.assertTrue(isClickable(element, driver, wait), String.format("Element %s is not clickable", elementPath));
    }

    public static void AssertExactText (WebDriver driver, Config elementConfig) {
        logger.info("AssertExactText");
        String elementPath = elementConfig.getString("textElement");
        String elementText = elementConfig.getString("exactText");
        logger.info(String.format("Checking element exact text %s", elementPath));

        int wait = 15;
        if(elementConfig.hasPath("wdw")) {
            wait = elementConfig.getInt("wdw");
        }
        WebDriverWait wdw = new WebDriverWait(driver, wait);
        wdw.until(ExpectedConditions.presenceOfElementLocated(By.xpath(elementPath)));

        String actualText = driver.findElement(By.xpath(elementPath)).getText();

        Assert.assertEquals(actualText, elementText, String.format("Element %s text should be %s but is %s", elementPath, elementText, actualText));
    }

    public static void SoftAssertExactText (WebDriver driver, Config elementConfig, SoftAssert softAssertions) {
        logger.info("SoftAssertExactText");
        String elementPath = elementConfig.getString("textElement");
        String elementText = elementConfig.getString("softExactText");
        logger.info(String.format("Checking element exact text %s", elementPath));

        int wait = 15;
        if(elementConfig.hasPath("wdw")) {
            wait = elementConfig.getInt("wdw");
        }
        WebDriverWait wdw = new WebDriverWait(driver, wait);
        wdw.until(ExpectedConditions.presenceOfElementLocated(By.xpath(elementPath)));

        String actualText = driver.findElement(By.xpath(elementPath)).getText();
        softAssertions.assertEquals(actualText, elementText, String.format("Element %s text should be %s but is %s", elementPath, elementText, actualText));
    }

    public static void AssertContainsText (WebDriver driver, Config elementConfig) {
        logger.info("AssertContainsText");
        String elementPath = elementConfig.getString("textElement");
        String elementText = elementConfig.getString("containsText");
        logger.info(String.format("Checking element exact text %s", elementPath));

        int wait = 15;
        if(elementConfig.hasPath("wdw")) {
            wait = elementConfig.getInt("wdw");
        }
        WebDriverWait wdw = new WebDriverWait(driver, wait);
        wdw.until(ExpectedConditions.presenceOfElementLocated(By.xpath(elementPath)));

        String actualText = driver.findElement(By.xpath(elementPath)).getText();

        Assert.assertTrue(actualText.contains(elementText), String.format("Element %s text should contains %s but is %s", elementPath, elementText, actualText));
    }

    public static void SoftAssertContainsText (WebDriver driver, Config elementConfig, SoftAssert softAssertions) {
        logger.info("SoftAssertContainsText");
        String elementPath = elementConfig.getString("textElement");
        String elementText = elementConfig.getString("softContainsText");
        logger.info(String.format("Checking element exact text %s", elementPath));

        int wait = 15;
        if(elementConfig.hasPath("wdw")) {
            wait = elementConfig.getInt("wdw");
        }
        WebDriverWait wdw = new WebDriverWait(driver, wait);
        wdw.until(ExpectedConditions.presenceOfElementLocated(By.xpath(elementPath)));

        String actualText = driver.findElement(By.xpath(elementPath)).getText();
        softAssertions.assertTrue(actualText.contains(elementText), String.format("Element %s text should contains %s but is %s", elementPath, elementText, actualText));
    }

    public static void AssertCSSValue(WebDriver driver, Config config) {
        logger.info("Assert CSS Value");
        String XPath = config.getString("assertCSS");
        String CSSValue = config.getString("CSSValue");
        String CSSField = config.getString("CSSField");
        logger.info(driver.findElement(By.xpath(XPath)).getCssValue(CSSField));
        boolean CSSValueFound = driver.findElement(By.xpath(XPath)).getCssValue(CSSField).equals(CSSValue);
        Assert.assertTrue(CSSValueFound,String.format("Element with CSS field %s and value %s not found",CSSField,CSSValue));
        logger.info("Assert Values - END");
    }

    public static void SoftAssertCSSValue(WebDriver driver, Config config, SoftAssert softAssertions) {
        logger.info("Soft Assert CSS Value");
        String XPath = config.getString("softAssertCSS");
        String CSSValue = config.getString("CSSValue");
        String CSSField = config.getString("CSSField");
        logger.info(driver.findElement(By.xpath(XPath)).getCssValue(CSSField));
        boolean CSSValueFound = driver.findElement(By.xpath(XPath)).getCssValue(CSSField).equals(CSSValue);
        softAssertions.assertTrue(CSSValueFound,String.format("Element with CSS field %s and value %s not found",CSSField,CSSValue));
        logger.info("Soft Assert Values - END");
    }


    public static boolean isClickable(WebElement el, WebDriver driver, int w)
    {
        try{
            WebDriverWait wait = new WebDriverWait(driver, w);
            wait.until(ExpectedConditions.elementToBeClickable(el));
            return el.isEnabled();
        }
        catch (Exception e){
            return false;
        }
    }

    public static boolean exists(String path, WebDriver driver)
    {
        try{
            WebDriverWait wait = new WebDriverWait(driver, 10);
            wait.until(ExpectedConditions.presenceOfElementLocated(By.xpath(path)));
            return true;
        }
        catch (Exception e){
            return false;
        }
    }
}
