package cz.atos.Selenium.base;

import com.typesafe.config.Config;
import com.typesafe.config.ConfigValueFactory;
import org.apache.log4j.Logger;
import org.openqa.selenium.*;
import org.openqa.selenium.support.ui.ExpectedConditions;
import org.openqa.selenium.support.ui.WebDriverWait;
import org.testng.asserts.SoftAssert;

import java.awt.*;
import java.awt.datatransfer.StringSelection;
import java.awt.event.KeyEvent;
import java.io.UnsupportedEncodingException;
import java.net.*;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import static cz.atos.Selenium.base.AssertMethods.*;
import static cz.atos.Selenium.base.ElementMethods.*;
import static cz.atos.Selenium.base.HelperMethods.*;
import static cz.atos.Selenium.base.MouseMethods.*;

public class BaseClass {

    private static final Logger logger = Logger.getLogger(BaseClass.class);

    public static void ActionsList(WebDriver driver, String browser, Config config, SoftAssert softAssertions) {
        if (config.hasPath("actionsList") && config.getStringList("actionsList").size() > 0) {
            List<String> actions = config.getStringList("actionsList");
            String mainWindow = driver.getWindowHandle();

            actions.forEach((action) -> {

                if (config.hasPath("actions." + action + ".sleep")) {
                    try {
                        int sleepTime = config.getInt("actions." + action + ".sleep");
                        logger.info(String.format("Sleep %2d ms ......zzz", sleepTime));
                        Thread.sleep(config.getInt("actions." + action + ".sleep"));
                    } catch (InterruptedException e) {
                        logger.error(e);
                        e.printStackTrace();
                    }
                }
                if (config.hasPath("actions." + action + ".switchWindows")) {
                    Set<String> handle = driver.getWindowHandles();
                    Iterator<String> it = handle.iterator();
                    String parent = it.next();
                    String child = it.next();
//                    for (String winHandle : driver.getWindowHandles()) {
//                        if (!mainWindow.equals(winHandle)) {
//                            logger.info("Handles switched...");
//                            if (!driver.getWindowHandle().equals(mainWindow)) {
//                                driver.switchTo().defaultContent();
//                            } else {
//                            driver.switchTo().window(winHandle);
//                            }
//                            logger.info("New handle focused...");
//                        }
//                    }
                    logger.info("Handles switched...");
                    driver.switchTo().window(child);
                    logger.info("New handle focused...");

                }
                if (config.hasPath("actions." + action + ".url")) {
                    logger.info("Browser: " + browser);
                    logger.info("Redirect to " + config.getString("actions." + action + ".url"));
                    String url = config.getString("actions." + action + ".url");
                    if ("chrome".equals(browser) || "edge".equals(browser)) {
                        if (config.hasPath("actions." + action + ".user") && config.hasPath("actions." + action + ".password")) {
                            String user = null;
                            String password = null;
                            try {
                                user = URLEncoder.encode(config.getString("actions." + action + ".user"), "utf-8");
                                password = URLEncoder.encode(config.getString("actions." + action + ".password"),"utf-8");
                            } catch (UnsupportedEncodingException e) {
                                logger.error(e);
                                e.printStackTrace();
                            }
                            try {
                                URL uri = new URL(config.getString("actions." + action + ".url"));
                                url = uri.getProtocol() + "://" + user + ":" + password + "@" + uri.getAuthority() + uri.getPath();
                            } catch (MalformedURLException e) {
                                logger.error(e);
                                e.printStackTrace();
                            }
                        }
                        driver.navigate().to(url);
                    } else {
                        driver.navigate().to(url);

                        if (config.hasPath("actions." + action + ".password")) {
                            try {
                                Robot rb = new Robot();
                                StringSelection password = new StringSelection(config.getString("actions." + action + ".password"));
                                Toolkit.getDefaultToolkit().getSystemClipboard().setContents(password, null);
                                rb.keyPress(KeyEvent.VK_CONTROL);
                                rb.keyPress(KeyEvent.VK_V);
                                rb.keyRelease(KeyEvent.VK_V);
                                rb.keyRelease(KeyEvent.VK_CONTROL);

                                rb.keyPress(KeyEvent.VK_ENTER);
                                rb.keyRelease(KeyEvent.VK_ENTER);
                            } catch (AWTException e) {
                                logger.error(e);
                                e.printStackTrace();
                            }
                        }
                    }
                    logger.info("After login...");
                    WebDriverWait wdw = new WebDriverWait(driver, 60);
                    long startTime = System.currentTimeMillis();
                    logger.info("Waiting for url to be " + config.getString("actions." + action + ".url"));
                    wdw.until(ExpectedConditions.urlToBe(url));
                    logger.info("Waiting time to load: " + (System.currentTimeMillis() - startTime) + "ms");
                }
                if (config.hasPath("actions." + action + ".input") && config.hasPath("actions." + action + ".value")) {
                    FillInputIfExists(driver, config.getConfig("actions." + action));
                }
                if (config.hasPath("actions." + action + ".textarea") && config.hasPath("actions." + action + ".value")) {
                    FillTextareaIfExists(driver, config.getConfig("actions." + action));
                }
                if (config.hasPath("actions." + action + ".select")) {
                    FillSelectIfExists(driver, config.getConfig("actions." + action));
                }
                if (config.hasPath("actions." + action + ".radio")) {
                    FillRadioIfExists(driver, config.getConfig("actions." + action));
                }
                if (config.hasPath("actions." + action + ".checkbox")) {
                    FillCheckboxIfExists(driver, config.getConfig("actions." + action));
                }
                if (config.hasPath("actions." + action + ".waitElement")) {
                    WaitFor(driver, config.getConfig("actions." + action));
                }
                if (config.hasPath("actions." + action + ".button")) {
                    try {
                        ClickOnButtonIfExists(driver, config.getConfig("actions." + action));
                    } catch (InterruptedException e) {
                        logger.info(e);
                        e.printStackTrace();
                    }
                }
                if (config.hasPath("actions." + action + ".CSSButton")) {
                    try {
                        ClickOnButtonIfCSSExists(driver, config.getConfig("actions." + action));
                    } catch (InterruptedException e) {
                        logger.info(e);
                        e.printStackTrace();
                    }
                }
                if (config.hasPath("actions." + action + ".dblClick")) {
                    try {
                        DblClickOnButtonIfExists(driver, config.getConfig("actions." + action));
                    } catch (InterruptedException e) {
                        logger.error(e);
                        e.printStackTrace();
                    }
                }
                if (config.hasPath("actions." + action + ".rightClick")) {
                    try {
                        RightClickOnButtonIfExists(driver, config.getConfig("actions." + action));
                    } catch (InterruptedException e) {
                        logger.error(e);
                        e.printStackTrace();
                    }
                }
                if (config.hasPath("actions." + action + ".fileName")) {
                    try {
                        UploadFile(driver, config.getConfig("actions." + action));
                    } catch (InterruptedException e) {
                        logger.error(e);
                        e.printStackTrace();
                    }
                }
                if (config.hasPath("actions." + action + ".alertAction")) {
                    AlertHandle(driver, config.getConfig("actions." + action));
                }
                if (config.hasPath("actions." + action + ".softImg")) {
                    SoftAssertImgExists(driver, config.getConfig("actions." + action + ".softImg"), softAssertions);
                }
                if (config.hasPath("actions." + action + ".softImgXPath")) {
                    SoftAssertImgExistsByXPath(driver, config.getConfig("actions." + action + ".softImgXPath"), softAssertions);
                }
                if (config.hasPath("actions." + action + ".softPopup")) {
                    String popupExpectedText = "";
                    if (config.hasPath("actions." + action + ".softPopupText")) {
                        popupExpectedText = config.getString("actions." + action + ".softPopupText");
                    }
                    SoftAssertPopupDisplayed(driver, config.getString("actions." + action + ".softPopup"), popupExpectedText, softAssertions);
                }
                if ((config.hasPath("actions." + action + ".softAssertValuesList")) && config.getStringList("actions." + action + ".softAssertValuesList").size() > 0) {
                    SoftAssertValues(driver, config.getConfig("actions." + action), softAssertions);
                }
                if ((config.hasPath("actions." + action + ".hardAssertValuesList")) && config.getStringList("actions." + action + ".hardAssertValuesList").size() > 0) {
                    HardAssertValues(driver, config);
                }
                if (config.hasPath("actions." + action + ".assertTable")) {
                    AssertTable(driver, config.getConfig("actions." + action + ".assertTable"));
                }
                if (config.hasPath("actions." + action + ".softAssertTable")) {
                    SoftAssertTable(driver, config.getConfig("actions." + action + ".softAssertTable"), softAssertions);
                }
                if (config.hasPath("actions." + action + ".exists")) {
                    AssertExists(driver, config.getString("actions." + action + ".exists"));
                }
                if (config.hasPath("actions." + action + ".softExists")) {
                    SoftAssertExists(driver, config.getString("actions." + action + ".softExists"), softAssertions);
                }
                if (config.hasPath("actions." + action + ".displayed")) {
                    AssertDisplayed(driver, config.getString("actions." + action + ".displayed"));
                }
                if (config.hasPath("actions." + action + ".softDisplayed")) {
                    SoftAssertDisplayed(driver, config.getString("actions." + action + ".softDisplayed"), softAssertions);
                }
                if (config.hasPath("actions." + action + ".clickable")) {
                    AssertClickable(driver, config.getConfig("actions." + action ));
                }
                if (config.hasPath("actions." + action + ".softClickable")) {
                    SoftAssertClickable(driver, config.getConfig("actions." + action ), softAssertions);
                }
                if (config.hasPath("actions." + action + ".exactText")) {
                    AssertExactText(driver, config.getConfig("actions." + action ));
                }
                if (config.hasPath("actions." + action + ".softExactText")) {
                    SoftAssertExactText(driver, config.getConfig("actions." + action ), softAssertions);
                }
                if (config.hasPath("actions." + action + ".containsText")) {
                    AssertContainsText(driver, config.getConfig("actions." + action ));
                }
                if (config.hasPath("actions." + action + ".softContainsText")) {
                    SoftAssertContainsText(driver, config.getConfig("actions." + action ), softAssertions);
                }
                if (config.hasPath("actions." + action + ".assertCSS")) {
                    AssertCSSValue(driver, config.getConfig("actions." + action ));
                }
                if (config.hasPath("actions." + action + ".softAssertCSS")) {
                    SoftAssertCSSValue(driver, config.getConfig("actions." + action ), softAssertions);
                }
            });
        }
    }
}
