package cz.atos.Selenium.base;

import com.typesafe.config.Config;
import org.apache.log4j.Logger;
import org.openqa.selenium.By;
import org.openqa.selenium.JavascriptExecutor;
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.WebElement;
import org.openqa.selenium.support.ui.ExpectedConditions;
import org.openqa.selenium.support.ui.WebDriverWait;


public class ElementMethods {
    private static final Logger logger = Logger.getLogger(ElementMethods.class);
    public static void FillInputIfExists(WebDriver driver, Config input) {
        logger.info("FillInputIfExists");
        int wait = 15;
        if (input.hasPath("wdw")) {
            wait = input.getInt("wdw");
        }
        WebDriverWait wdw = new WebDriverWait(driver, wait);
        JavascriptExecutor js = (JavascriptExecutor) driver;
        String inputXpath = input.getString("input");


        wdw.until(ExpectedConditions.presenceOfElementLocated(By.xpath(inputXpath)));
        String inputValue = input.getString("value");

        logger.info(inputXpath + " - " + inputValue);

        WebElement inputElement = driver.findElement(By.xpath(inputXpath));

        String event = null;

        if (input.hasPath("event")) {
            event = input.getString("event");
        }

        if (event != null) {
            js.executeScript("var event = document.createEvent('Event'); event.initEvent('" + event + "', false, true); arguments[0].value ='" + inputValue + "'; arguments[0].dispatchEvent(event); return true", inputElement);
        } else {
            js.executeScript("arguments[0].value ='" + inputValue + "'; arguments[0].focus(); arguments[0].blur(); return true", inputElement);
        }
    }

    public static void FillSelectIfExists(WebDriver driver, Config select) {
        logger.info("FillSelectIfExists");
        int wait = 15;
        if (select.hasPath("wdw")) {
            wait = select.getInt("wdw");
        }
        WebDriverWait wdw = new WebDriverWait(driver, wait);
        JavascriptExecutor js = (JavascriptExecutor) driver;
        String selectXpath = select.getString("select");

        wdw.until(ExpectedConditions.presenceOfElementLocated(By.xpath(selectXpath)));
        String inputValue = select.getString("value");
        logger.info(selectXpath + " - " + inputValue);

        WebElement inputElement = driver.findElement(By.xpath(selectXpath));
        String event = null;

        if (select.hasPath("event")) {
            event = select.getString("event");
        }

        if (event != null) {
            js.executeScript("var event = document.createEvent('Event'); event.initEvent('" + event + "', false, true); arguments[0].value ='" + inputValue + "'; arguments[0].dispatchEvent(event); return true", inputElement);
        } else {
            js.executeScript("arguments[0].value ='" + inputValue + "'; arguments[0].focus(); arguments[0].blur(); return true", inputElement);
        }
    }

    public static void FillRadioIfExists(WebDriver driver, Config radio) {
        logger.info("FillRadioIfExists");
        int wait = 15;
        if (radio.hasPath("wdw")) {
            wait = radio.getInt("wdw");
        }
        WebDriverWait wdw = new WebDriverWait(driver, wait);
        JavascriptExecutor js = (JavascriptExecutor) driver;
        String radioXpath = radio.getString("radio");

        wdw.until(ExpectedConditions.presenceOfElementLocated(By.xpath(radioXpath)));
        logger.info(radioXpath);

        WebElement inputElement = driver.findElement(By.xpath(radioXpath));

        String event = null;

        if (radio.hasPath("event")) {
            event = radio.getString("event");
        }

        if (event != null) {
            if ("click".equals(event)) {
                js.executeScript("var event = document.createEvent('MouseEvent'); event.initMouseEvent('" + event + "', true,true,window,0,0,0,0,0,false,false,false,false,0,null); arguments[0].dispatchEvent(event); return true", inputElement);

            } else {
                js.executeScript("var event = document.createEvent('Event'); event.initEvent('" + event + "', false, true); arguments[0].checked = true; arguments[0].dispatchEvent(event); return true", inputElement);
            }
        } else {
            js.executeScript("arguments[0].checked = true; arguments[0].blur(); return true", inputElement);
        }
    }

    public static void FillCheckboxIfExists(WebDriver driver, Config checkbox) {
        logger.info("FillCheckboxIfExists");
        int wait = 15;
        if (checkbox.hasPath("wdw")) {
            wait = checkbox.getInt("wdw");
        }
        WebDriverWait wdw = new WebDriverWait(driver, wait);
        JavascriptExecutor js = (JavascriptExecutor) driver;
        String selectXpath = checkbox.getString("checkbox");

        wdw.until(ExpectedConditions.presenceOfElementLocated(By.xpath(selectXpath)));
        logger.info(selectXpath);

        WebElement checkboxElement = driver.findElement(By.xpath(selectXpath));

        js.executeScript("var event = document.createEvent('Event'); event.initEvent('click', false, true);  arguments[0].dispatchEvent(event); return true", checkboxElement);
    }

    public static void FillTextareaIfExists(WebDriver driver, Config textarea) {
        logger.info("FillTextareaIfExists");
        int wait = 15;
        if (textarea.hasPath("wdw")) {
            wait = textarea.getInt("wdw");
        }
        WebDriverWait wdw = new WebDriverWait(driver, wait);
        JavascriptExecutor js = (JavascriptExecutor) driver;
        String textareaXpath = textarea.getString("textarea");

        wdw.until(ExpectedConditions.presenceOfElementLocated(By.xpath(textareaXpath)));
        String textareaValue = textarea.getString("value");
        logger.info(textareaXpath + " - " + textareaValue);

        WebElement textareaElement = driver.findElement(By.xpath(textareaXpath));

        String event = null;

        if (textarea.hasPath("event")) {
            event = textarea.getString("event");
        }

        if (event != null) {
            js.executeScript("var event = document.createEvent('Event'); event.initEvent('" + event + "', false, true); arguments[0].value ='" + textareaValue + "'; arguments[0].dispatchEvent(event); return true", textareaElement);
        } else {
            js.executeScript("arguments[0].value ='" + textareaValue + "'; arguments[0].focus(); arguments[0].blur(); return true", textareaElement);
        }
    }
}
