package cz.atos.Selenium.base;

import com.typesafe.config.Config;
import org.apache.log4j.Logger;
import org.openqa.selenium.By;
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.WebElement;
import org.openqa.selenium.support.ui.ExpectedConditions;
import org.openqa.selenium.support.ui.WebDriverWait;

import java.io.File;

public class HelperMethods {
    private static final Logger logger = Logger.getLogger(HelperMethods.class);
    public static void WaitFor (WebDriver driver, Config config) {
        logger.info("WaitFor");
        int wait = 15;
        if(config.hasPath("wdw")) {
            wait = config.getInt("wdw");
        }
        WebDriverWait wdw = new WebDriverWait(driver, wait);
        String waitElement = config.getString("waitElement");
        String waitTo = config.getString("waitTo");
        logger.info(waitElement + " to be " + waitTo);
        if ("invisible".equals(waitTo)) {
            wdw.until(ExpectedConditions.invisibilityOf(driver.findElement(By.xpath(waitElement))));
        }
        if ("visible".equals(waitTo)) {
            wdw.until(ExpectedConditions.visibilityOf(driver.findElement(By.xpath(waitElement))));
        }
        if ("clickable".equals(waitTo)) {
            wdw.until(ExpectedConditions.elementToBeClickable(driver.findElement(By.xpath(waitElement))));
        }
    }

    public static void UploadFile (WebDriver driver, Config config) throws InterruptedException {
        logger.info("UploadFile");
        int wait = 15;
        if(config.hasPath("wdw")) {
            wait = config.getInt("wdw");
        }
        WebDriverWait wdw = new WebDriverWait(driver, wait);

        String filename = config.getString("fileName");

        File file = new File("testConfig/files/" + filename);
        String filePath = file.getAbsolutePath();

        String inputElement = config.getString("uploadElement");

        wdw.until(ExpectedConditions.presenceOfElementLocated(By.xpath(inputElement)));
        WebElement uploadElement = driver.findElement(By.xpath(inputElement));
        uploadElement.sendKeys(filePath);

        Thread.sleep(500);
    }

    public static void AlertHandle (WebDriver driver, Config config) {
        logger.info("AlertHandle");
        switch (config.getString("alertAction")) {
            case "accept":
                driver.switchTo().alert().accept();
                break;
            case "dismiss":
                driver.switchTo().alert().dismiss();
        }

    }
}
